import logging
from typing import List, Optional

from .object import Object

log: logging.Logger = logging.getLogger(__name__)


class Loadout(Object):
    """
    Represents a Call of Duty loadout object.

    Parameters
    ----------
    name : str
        Name of the loadout.
    primary : callofduty.LoadoutWeapon
        Primary weapon slot of the loadout.
    secondary : callofduty.LoadoutWeapon
        Secondary weapon slot of the loadout.
    equipment : list
        Array of LoadoutItem objects of the loadout's equipment.
    perks : list
        Array of LoadoutItem objects of the loadout's perks.
    wildcards : list
        Array of LoadoutItem objects of the loadout's wildcards.
    unlocked : bool
        Boolean value indicating whether the loadout slot is unlocked.
    """

    _type: str = "Loadout"

    def __init__(self, client, data: dict):
        super().__init__(client)

        self.name: str = data.pop("customClassName")
        self.primary: LoadoutWeapon = LoadoutWeapon(self, data.pop("primaryWeapon"))
        self.secondary: LoadoutWeapon = LoadoutWeapon(self, data.pop("secondaryWeapon"))
        self.equipment: List[LoadoutItem] = []
        self.perks: List[LoadoutItem] = []
        self.wildcards: List[LoadoutItem] = []
        self.unlocked: bool = data.pop("unlocked")

        if (_equipment := data.pop("equipment")) is not None:
            self.equipment.append(LoadoutItem(self, _equipment))

        if (_gear := data.pop("gear")) is not None:
            self.equipment.append(LoadoutItem(self, _gear))

        for _perk in data.pop("perks"):
            self.perks.append(LoadoutItem(self, _perk))

        for _wildcard in data.pop("wildcards"):
            self.wildcards.append(LoadoutItem(self, _wildcard))


class LoadoutWeapon(Object):
    """
    Represents a Call of Duty loadout weapon object.

    Parameters
    ----------
    id : str
        ID of the weapon.
    variant : str, optional
        ID of the weapon variant (default is None.)
    attachments : list
        Array of LoadoutItem objects for the weapon's attachments.
    camo : bool
        Boolean value indicating whether a camo is equipped to the weapon.
    """

    _type: str = "LoadoutWeapon"

    def __init__(self, client, data: dict):
        super().__init__(client)

        self.id: str = data.pop("id")
        self.variant: Optional[str] = None
        self.attachments: List[LoadoutItem] = []
        self.camo: bool = data.pop("camoEquipped")

        if (_variant := data.pop("variant")) is not None:
            self.variant: Optional[str] = _variant["id"]

        # Optics and Operator Mods are attachments, there's no reason to
        # seperate them from the attachments array.
        # This is also to (hopefully) make Modern Warfare support easier.
        if (_optic := data.pop("optic")) is not None:
            self.attachments.append(LoadoutItem(self, _optic))

        if (_opMod := data.pop("operatorMod")) is not None:
            self.attachments.append(LoadoutItem(self, _opMod))

        if (_attachments := data.pop("attachments")) is not None:
            for _attachment in _attachments:
                self.attachments.append(LoadoutItem(self, _attachment))


class LoadoutItem(Object):
    """
    Represents a Call of Duty loadout item object.

    Parameters
    ----------
    id : str
        ID of the item.
    """

    _type: str = "LoadoutItem"

    def __init__(self, client, data: dict):
        super().__init__(client)

        self.id: str = data.pop("id")



import logging
from typing import Dict, List, Union

from .enums import Mode, Platform, Title
from .object import Object
from .player import Player

log: logging.Logger = logging.getLogger(__name__)


class AuthenticityStamp(Object):
    """
    Represents a Call of Duty Authenticity Stamp object.

    Parameters
    ----------
    platform : callofduty.Platform
        Platform of the player.
    username : str
        Player's username for the designated platform.
    title : callofduty.Title
        Title which the match took place.
    mode : callofduty.Mode
        Call of Duty mode which the match took place.
    players : list
        Array of Player objects for players present when the match ended.
    playersLeft : list
        Array of Player objects for players who left the match.
    data : dict
        JSON data for the requested Authenticity Stamp.
    settings : dict
        JSON data for the game settings of the requested Authenticity Stamp.
    stats : dict
        JSON data for the statistics of the player of the requested Authenticity Stamp.
    """

    _type: str = "AuthenticityStamp"

    def __init__(self, client, data: dict):
        super().__init__(client)

        self.platform: Platform = Platform(data.pop("platform"))
        self.username: str = data.pop("username")
        self.title: Title = Title(data.pop("title"))
        self.mode: Mode = Mode(data.pop("mode"))
        self.players: List[Player] = []
        self.playersLeft: List[Player] = []
        self.data: Dict[str, Union[int, str, bool, None]] = {}
        self.settings: Dict[str, Union[float, bool]] = data.pop("gameSettings")
        self.stats: Dict[str, Union[float]] = data.pop("playerStats")

        for _player in data.pop("partyMembers"):
            self.players.append(
                Player(self, {"platform": self.platform, "username": _player})
            )

        for _player in data.pop("partyMembersLeft"):
            self.players.append(
                Player(self, {"platform": self.platform, "username": _player})
            )

        for key in data:
            if (isinstance(data[key], dict) is False) and (isinstance(data[key], list) is False):
                self.data[key] = data[key]



"""
CallofDuty.py is an asynchronous, object-oriented Python wrapper for
the Call of Duty API.

GitHub: https://github.com/EthanC/CallofDuty.py
"""

import logging

from .auth import Login
from .client import Client
from .enums import *
from .errors import *
from .leaderboard import Leaderboard, LeaderboardEntry
from .loadout import Loadout, LoadoutItem, LoadoutWeapon
from .loot import LootItem, Season
from .match import Match
from .player import Player
from .squad import Squad
from .stamp import AuthenticityStamp

try:
    from logging import NullHandler
except ImportError:

    class NullHandler(logging.Handler):
        def emit(self, record):
            pass


logging.getLogger(__name__).addHandler(NullHandler())



from enum import Enum


class Platform(Enum):
    Activision = "uno"
    PlayStation = "psn"
    Xbox = "xbl"
    Steam = "steam"
    BattleNet = "battle"


class Title(Enum):
    ModernWarfare = "mw"
    BlackOps4 = "bo4"
    WWII = "wwii"
    InfiniteWarfare = "iw"
    BlackOps3 = "bo3"


class Mode(Enum):
    Multiplayer = "mp"
    Zombies = "zm"
    Blackout = "wz"


class Language(Enum):
    English = "en"
    French = "fr"
    German = "de"
    Italian = "it"
    Spanish = "es"


class TimeFrame(Enum):
    AllTime = "alltime"
    Monthly = "monthly"
    Weekly = "weekly"


class GameType(Enum):
    Core = "core"
    Hardcore = "hc"
    WorldLeague = "arena"



import logging
import urllib.parse
from typing import Dict, Optional, Union

from httpx import AsyncClient, Response

from .errors import Forbidden, HTTPException, NotFound

log: logging.Logger = logging.getLogger(__name__)


async def JSONorText(res: Response) -> Union[dict, str]:
    """
    Determine the media type of the provided response.

    Parameters
    ----------
    res : httpx.Response
        Response object to determine media type.

    Returns
    -------
    object
        If media type is JSON data, return dict. Otheriwse return data as str.
    """

    if res.headers["Content-Type"].lower() == "application/json;charset=utf-8":
        return res.json()
    else:
        return res.text()


class Request:
    """
    Represents a Request object.

    Parameters
    ----------
    method : str
        HTTP method to perform for the reqest.
    endpoint : str, optional
        Endpoint to execute the request on (default is None.)
    baseUrl : str, optional
        Base URL to use for the request (default is https://callofduty.com/)
    headers : dict, optional
        Headers to send along with the request (default is None.)
    """

    defaultBaseUrl: str = "https://callofduty.com/"
    squadsBaseUrl: str = "https://squads.callofduty.com/"

    accessToken: Optional[str] = None
    deviceId: Optional[str] = None

    def __init__(self, method: str, endpoint: Optional[str] = None, **kwargs):
        self.method: str = method
        self.headers: Dict[str, str] = {}

        if endpoint is not None:
            baseUrl: str = kwargs.get("baseUrl", self.defaultBaseUrl)
            self.url: str = f"{baseUrl}{endpoint}"

        headers: Optional[Dict[str, str]] = kwargs.get("headers")
        if isinstance(headers, dict):
            self.headers.update(headers)

    def SetHeader(self, key: str, value: str):
        self.headers[key] = value


class HTTP:
    """HTTP client used to communicate with the Call of Duty API."""

    def __init__(self, auth):
        self.auth = auth
        self.session: AsyncClient = auth.session

    async def Send(self, req: Request) -> Union[dict, str]:
        """
        Perform an HTTP request.

        Parameters
        ----------
        req : callofduty.HTTP.Request
            Object representing the HTTP request.

        Returns
        -------
        dict/str
            Response of the HTTP request.
        """

        req.SetHeader("Authorization", f"Bearer {self.auth.AccessToken}")
        req.SetHeader("x_cod_device_id", self.auth.DeviceId)

        async with self.session as client:
            res: Response = await client.request(
                req.method, req.url, headers=req.headers
            )

            data: Union[dict, str] = await JSONorText(res)
            if isinstance(data, dict):
                status: Optional[str] = data.get("status")

                # The API tends to return HTTP 200 even when an error occurs
                if status == "error":
                    raise HTTPException(res.status_code, data)

            # HTTP 2XX: Success
            if 300 > res.status_code >= 200:
                return data

            # HTTP 429: Too Many Requests
            if res.status_code == 429:
                # TODO Handle rate limiting
                raise HTTPException(res.status_code, data)

            # HTTP 500/502: Internal Server Error/Bad Gateway
            if res.status_code == 500 or res.status_code == 502:
                # TODO Handle Unconditional retries
                raise HTTPException(res.status_code, data)

            # HTTP 403: Forbidden
            if res.status_code == 403:
                raise Forbidden(res.status_code, data)
            # HTTP 404: Not Found
            elif res.status_code == 404:
                raise NotFound(res.status_code, data)
            else:
                raise HTTPException(res.status_code, data)

    async def GetAppLocalize(self, language: str) -> Union[dict, str]:
        return await self.Send(
            Request(
                "GET",
                f"content/atvi/callofduty/mycod/web/{language}/data/json/iq-content-xapp.js",
            )
        )

    async def GetWebLocalize(self, language: str) -> Union[dict, str]:
        return await self.Send(
            Request(
                "GET",
                f"content/atvi/callofduty/mycod/web/{language}/data/json/iq-content-xweb.js",
            )
        )

    async def GetNewsFeed(self, language: str) -> Union[dict, str]:
        return await self.Send(Request("GET", f"site/cod/franchiseFeed/{language}"))

    async def GetFriendFeed(self) -> Union[dict, str]:
        return await self.Send(
            Request("GET", "api/papi-client/userfeed/v1/friendFeed/rendered/")
        )

    async def GetMyIdentities(self) -> Union[dict, str]:
        return await self.Send(Request("GET", "api/papi-client/crm/cod/v2/identities/"))

    async def GetMyAccounts(self) -> Union[dict, str]:
        return await self.Send(Request("GET", "api/papi-client/crm/cod/v2/accounts/"))

    async def GetMyFriends(self) -> Union[dict, str]:
        return await self.Send(
            Request("GET", "api/papi-client/codfriends/v1/compendium")
        )

    async def SearchPlayer(self, platform: str, username: str) -> Union[dict, str]:
        return await self.Send(
            Request(
                "GET",
                f"api/papi-client/crm/cod/v2/platform/{platform}/username/{urllib.parse.quote(username)}/search",
            )
        )

    async def GetPlayerProfile(
        self, platform: str, username: str, title: str, mode: str
    ) -> Union[dict, str]:
        return await self.Send(
            Request(
                "GET",
                f"api/papi-client/stats/cod/v1/title/{title}/platform/{platform}/gamer/{urllib.parse.quote(username)}/profile/type/{mode}",
            )
        )

    async def GetPlayerMatches(
        self,
        platform: str,
        username: str,
        title: str,
        mode: str,
        limit: int,
        startTimestamp: int,
        endTimeStamp: int,
    ) -> Union[dict, str]:
        return await self.Send(
            Request(
                "GET",
                f"api/papi-client/crm/cod/v2/title/{title}/platform/{platform}/gamer/{urllib.parse.quote(username)}/matches/{mode}/start/{startTimestamp}/end/{endTimeStamp}?limit={limit}",
            )
        )

    async def GetPlayerMatchesDetailed(
        self,
        platform: str,
        username: str,
        title: str,
        mode: str,
        limit: int,
        startTimestamp: int,
        endTimeStamp: int,
    ) -> Union[dict, str]:
        return await self.Send(
            Request(
                "GET",
                f"api/papi-client/crm/cod/v2/title/{title}/platform/{platform}/gamer/{urllib.parse.quote(username)}/matches/{mode}/start/{startTimestamp}/end/{endTimeStamp}/details?limit={limit}",
            )
        )

    async def GetMatch(
        self, title: str, platform: str, matchId: int
    ) -> Union[dict, str]:
        return await self.Send(
            Request(
                "GET",
                f"api/papi-client/ce/v1/title/{title}/platform/{platform}/match/{matchId}/matchMapEvents",
            )
        )

    async def GetLeaderboard(
        self,
        title: str,
        platform: str,
        gameType: str,
        gameMode: str,
        timeFrame: str,
        page: int,
    ) -> Union[dict, str]:
        return await self.Send(
            Request(
                "GET",
                f"api/papi-client/leaderboards/v2/title/{title}/platform/{platform}/time/{timeFrame}/type/{gameType}/mode/{gameMode}/page/{page}",
            )
        )

    async def GetPlayerLeaderboard(
        self,
        title: str,
        platform: str,
        username: str,
        gameType: str,
        gameMode: str,
        timeFrame: str,
    ) -> Union[dict, str]:
        return await self.Send(
            Request(
                "GET",
                f"api/papi-client/leaderboards/v2/title/{title}/platform/{platform}/time/{timeFrame}/type/{gameType}/mode/{gameMode}/gamer/{urllib.parse.quote(username)}",
            )
        )

    async def GetAvailableMaps(
        self, title: str, platform: str, mode: str
    ) -> Union[dict, str]:
        return await self.Send(
            Request(
                "GET",
                f"api/papi-client/ce/v1/title/{title}/platform/{platform}/gameType/{mode}/communityMapData/availability",
            )
        )

    async def GetLootSeason(
        self, title: str, season: int, platform: str, language: str
    ) -> Union[dict, str]:
        return await self.Send(
            Request(
                "GET",
                f"api/papi-client/loot/title/{title}/platform/{platform}/list/loot_season_{season}/{language}",
            )
        )

    async def GetPlayerLoadouts(
        self, platform: str, username: str, title: str, mode: str
    ) -> Union[dict, str]:
        return await self.Send(
            Request(
                "GET",
                f"api/papi-client/loadouts/v3/title/{title}/platform/{platform}/gamer/{urllib.parse.quote(username)}/mode/{mode}",
            )
        )

    async def GetAuthenticityStamp(
        self, platform: str, username: str, phrase: str, title: str
    ) -> Union[dict, str]:
        return await self.Send(
            Request(
                "GET",
                f"api/papi-client/zmauth/v1/title/{title}/platform/{platform}/gamer/{urllib.parse.quote(username)}/zombies/match/authenticated/phrase/{urllib.parse.quote(phrase)}",
            )
        )

    async def GetSquad(self, name: str) -> Union[dict, str]:
        return await self.Send(
            Request(
                "GET",
                f"api/v2/squad/lookup/name/{urllib.parse.quote(name)}",
                baseUrl=Request.squadsBaseUrl,
            )
        )

    async def GetPlayerSquad(self, platform: str, username: str) -> Union[dict, str]:
        return await self.Send(
            Request(
                "GET",
                f"api/v2/squad/lookup/platform/{platform}/gamer/{urllib.parse.quote(username)}",
                baseUrl=Request.squadsBaseUrl,
            )
        )

    async def GetMySquad(self) -> Union[dict, str]:
        return await self.Send(
            Request("GET", "api/v2/squad/lookup/mine/", baseUrl=Request.squadsBaseUrl)
        )

    async def JoinSquad(self, name: str) -> Union[dict, str]:
        return await self.Send(
            Request(
                "GET",
                f"api/v2/squad/join/{urllib.parse.quote(name)}",
                baseUrl=Request.squadsBaseUrl,
            )
        )

    async def LeaveSquad(self) -> Union[dict, str]:
        return await self.Send(
            Request("GET", "api/v2/squad/leave/", baseUrl=Request.squadsBaseUrl)
        )



import logging
from typing import List

from .enums import Platform, Title
from .object import Object
from .player import Player

log: logging.Logger = logging.getLogger(__name__)


class Match(Object):
    """
    Represents a Call of Duty match object.

    Parameters
    ----------
    id : int
        Match ID.
    platform : callofduty.Platform
        Platform of the player.
    title : callofduty.Title
        Title which the match took place.
    """

    _type: str = "Match"

    def __init__(self, client, data: dict):
        super().__init__(client)

        self.id: int = data.pop("id")
        self.platform: Platform = Platform(data.pop("platform"))
        self.title: Title = Title(data.pop("title"))

    async def teams(self) -> List[List[Player]]:
        """
        Get the teams which played in the match.

        Returns
        -------
        list
            Array containing two child arrays, one for each team. Each
            team's array contains Player objects which represent the
            players on the team.
        """

        return await self._client.GetMatchTeams(self.title, self.platform, self.id)

    async def details(self) -> dict:
        """
        Get the full details of the match.

        Returns
        -------
        dict
            JSON data containing the full details of the match.
        """

        return await self._client.GetMatchDetails(self.title, self.platform, self.id)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['callofduty']

package_data = \
{'': ['*']}

install_requires = \
['httpx>=0.11.1,<0.12.0']

setup_kwargs = {
    'name': 'callofduty.py',
    'version': '1.1.0',
    'description': 'CallofDuty.py is an asynchronous, object-oriented Python wrapper for the Call of Duty API.',
    'long_description': '<div align="center">\n\n![CallofDuty.py](https://i.imgur.com/HXy6Dkd.png)\n\n<a href="https://pypi.python.org/pypi/callofduty.py"><img src="https://img.shields.io/pypi/v/callofduty.py?label=Version&style=for-the-badge" /></a>\n<a href="https://pypi.python.org/pypi/callofduty.py"><img src="https://img.shields.io/pypi/dm/callofduty.py?style=for-the-badge" /></a>\n<a href="https://twitter.com/Mxtive"><img src="https://img.shields.io/twitter/follow/Mxtive?color=1da1f2&label=Twitter&style=for-the-badge" /></a>\n<a href="https://discord.gg/callofduty"><img src="https://img.shields.io/discord/136986169563938816?color=7289DA&label=Discord&style=for-the-badge" /></a>\n\n</div>\n\n# CallofDuty.py\n\nCallofDuty.py is an asynchronous, object-oriented Python wrapper for the Call of Duty API.\n\n## Features\n\n-   Asynchronous and Pythonic using `async` and `await`\n-   Type checks and editor completion using [Type Hints](https://www.python.org/dev/peps/pep-0484/)\n-   Object-oriented and predictable abstractions\n\n## Usage\n\nConstruct a new Call of Duty client, then use the various services on the client to access different parts of the Call of Duty API.\n\n### Installation\n\nCallofDuty.py requires Python 3.8 or greater. Once this requirement is met, simply install CallofDuty.py using pip (or [Poetry](https://python-poetry.org/docs/cli/#add))!\n\n```\npip install callofduty.py\n```\n\n### Example\n\nThe following is a complete example which demonstrates:\n\n-   Authenticating with the Call of Duty API\n-   Searching for a user\n-   Listing the first 3 search results\n-   Getting the Modern Warfare Multiplayer profile of the second result\n-   Displaying their basic statistics\n\n```py\nimport asyncio\n\nimport callofduty\nfrom callofduty import Mode, Platform, Title\n\n\nasync def main():\n    client = await callofduty.Login("YourEmail@email.com", "YourPassword")\n\n    results = await client.SearchPlayers(Platform.Activision, "Captain Price", limit=3)\n    for player in results:\n        print(f"{player.username} ({player.platform.name})")\n\n    me = results[1]\n    profile = await me.profile(Title.ModernWarfare, Mode.Multiplayer)\n\n    level = profile["level"]\n    kd = profile["lifetime"]["all"]["properties"]["kdRatio"]\n    wl = profile["lifetime"]["all"]["properties"]["wlRatio"]\n\n    print(f"\\n{me.username} ({me.platform.name})")\n    print(f"Level: {level}, K/D Ratio: {kd}, W/L Ratio: {wl}")\n\n    await client.Logout()\n\nasyncio.get_event_loop().run_until_complete(main())\n```\n\n## Releases\n\nCallofDuty.py follows [Semantic Versioning](https://semver.org/) for tagging releases of the project.\n\nChangelogs can be found on the [Releases](https://github.com/EthanC/CallofDuty.py/releases) page and follow the [Keep a Changelog](https://keepachangelog.com/) format.\n\n## Contributing\n\nThe goal is to cover the entirety of the Call of Duty API, so contributions are always welcome. The calling pattern is pretty well-established, so adding new methods is relatively straightforward. See [`CONTRIBUTING.md`](https://github.com/EthanC/CallofDuty.py/blob/master/.github/CONTRIBUTING.md) for details.\n\n## Thanks & Credits\n\n-   [Tustin](https://github.com/Tustin) - Call of Duty API Authorization Flow\n-   [Activision](https://www.activision.com/) - Call of Duty Logo & API Service\n',
    'author': 'EthanC',
    'author_email': 'EthanC@github.com',
    'maintainer': 'Josh',
    'maintainer_email': 'Tustin@github.com',
    'url': 'https://github.com/EthanC/CallofDuty.py',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import logging
from typing import List, Optional

from .object import Object
from .player import Player

log: logging.Logger = logging.getLogger(__name__)


class Squad(Object):
    """
    Represents a Call of Duty Squad object.

    Parameters
    ----------
    name : str
        Name of Squad.
    description : str, optional
        Description of Squad (default is None.)
    avatarUrl : str, optional
        Avatar URL of Squad (default is None.)
    created : str, optional
        Date of Squad creation (default is None.)
    new : bool, optional
        Boolean indication of whether or not the Squad was recently created (default is False.)
    private : bool, optional
        Boolean indication of whether or not the Squad is private (default is False.)
    points : int, optional
        Number of points which the Squad currently has (default is None.)
    owner : object, optional
        Player object representing the Squad owner (default is None.)
    members : list, optional
        Array of player objects representing the Squad members (default is an empty list.)
    """

    _type: str = "Squad"

    def __init__(self, client, data: dict):
        super().__init__(client)

        self.name: str = data.pop("name")
        self.description: Optional[str] = data.pop("description", None)
        self.avatarUrl: Optional[str] = data.pop("avatarUrl", None)
        self.created: Optional[str] = data.pop("created", None)
        self.new: bool = data.pop("newlyFormed", False)
        self.private: bool = data.pop("private", False)
        self.points: Optional[int] = data.pop("points", None)

        # The Squads endpoints do not follow the same structure as the rest,
        # so the following is a hacky solution to that problem...
        self.owner: Player = Player(
            client,
            {
                "platform": data["creator"]["platform"],
                "username": data["creator"]["gamerTag"],
                "accountId": data["creator"]["platformId"],
                "avatarUrl": data["creator"]["avatarUrl"],
            },
        )

        self.members: List[Player] = []
        for member in data["members"]:
            self.members.append(
                Player(
                    client,
                    {
                        "platform": member["platform"],
                        "username": member["gamerTag"],
                        "accountId": member["platformId"],
                        "avatarUrl": member["avatarUrl"],
                    },
                )
            )

    async def join(self):
        """Join the Call of Duty Squad."""

        await self._client.JoinSquad(self.name)



import logging
from typing import List, Optional

from .enums import Mode, Platform, Title
from .loadout import Loadout, LoadoutItem
from .object import Object

log: logging.Logger = logging.getLogger(__name__)


class Player(Object):
    """
    Represents a Call of Duty player object.

    Parameters
    ----------
    platform : callofduty.Platform
        Platform of the player.
    username : str
        Player's username for the designated platform.
    accountId : int, optional
        Account ID for the player's designated platform (default is None.)
    avatarUrl : str, optional
        Url which returns an image of the player's avatar (default is None.)
    online : bool, optional
        Boolean indicating whether or not the player is currently online (default is False.)
    identities : list, optional
        Array of Player objects containing the player's identities (default is an empty list.)
    """

    _type: str = "Player"

    def __init__(self, client, data: dict):
        super().__init__(client)

        self.platform: Platform = Platform(data.pop("platform"))
        self.username: str = data.pop("username")
        self.accountId: Optional[int] = data.pop("accountId", None)
        self.avatarUrl: Optional[str] = data.pop("avatarUrl", None)
        self.online: bool = data.pop("online", False)
        self.identities: List[Player] = data.pop("identities", [])

    async def profile(self, title: Title, mode: Mode) -> dict:
        """
        Get the Call of Duty player's profile for the specified title and mode.

        Parameters
        ----------
        title : callofduty.Title
            Call of Duty title to get the player's profile from.
        mode: callofduty.Mode
            Call of Duty mode to get the player's profile from.

        Returns
        -------
        dict
            JSON data of the player's complete profile for the requested
            title and mode.
        """

        return await self._client.GetPlayerProfile(
            self.platform, self.username, title, mode
        )

    async def matches(self, title: Title, mode: Mode, **kwargs) -> list:
        """
        Get the Call of Duty player's match history for the specified title and mode.

        Parameters
        ----------
        title : callofduty.Title
            Call of Duty title to get the player's matches from.
        mode: callofduty.Mode
            Call of Duty mode to get the player's matches from.
        limit : int, optional
            Number of matches which will be returned (default is 10.)
        startTimestamp : int, optional
            Unix timestamp representing the earliest time which a returned
            match should've occured (default is None.)
        endTimestamp : int, optional
            Unix timestamp representing the latest time which a returned
            match should've occured (default is None.)

        Returns
        -------
        list
            Array of Match objects.
        """

        return await self._client.GetPlayerMatches(
            self.platform, self.username, title, mode, **kwargs
        )

    async def matchesSummary(self, title: Title, mode: Mode, **kwargs) -> dict:
        """
        Get the Call of Duty player's match history for the specified title and mode.

        Parameters
        ----------
        title : callofduty.Title
            Call of Duty title to get the player's matches from.
        mode: callofduty.Mode
            Call of Duty mode to get the player's matches from.
        limit : int, optional
            Number of matches which will be returned (default is 10.)
        startTimestamp : int, optional
            Unix timestamp representing the earliest time which a returned
            match should've occured (default is None.)
        endTimestamp : int, optional
            Unix timestamp representing the latest time which a returned
            match should've occured (default is None.)

        Returns
        -------
        dict
            JSON data containing recent matches summary.
        """

        return await self._client.GetPlayerMatchesSummary(
            self.platform, self.username, title, mode, **kwargs
        )

    async def leaderboard(self, title: Title, **kwargs):
        """
        Get the specified Leaderboard page which contains the Call of Duty player.

        Parameters
        ----------
        title : callofduty.Title
            Call of Duty title which the leaderboard represents.
        gameType : callofduty.GameType, optional
            Game type to get the leaderboard for (default is Core.)
        gameMode : str, optional
            Game mode to get the leaderboard for (default is Career.)
        timeFrame : callofduty.TimeFrame, optional
            Time Frame to get the leaderboard for (default is All-Time.)

        Returns
        -------
        object
            Leaderboard object representing the specified details.
        """

        return await self._client.GetPlayerLeaderboard(
            title, self.platform, self.username, **kwargs
        )

    async def loadouts(self, title: Title, **kwargs) -> List[Loadout]:
        """
        Get the Call of Duty player's loadouts for the specified title and mode.

        Parameters
        ----------
        title : callofduty.Title
            Call of Duty title to get the player's loadouts from.
        mode: callofduty.Mode, optional
            Call of Duty mode to get the player's loadouts from (default is Multiplayer.)

        Returns
        -------
        list
            Array of loadout objects.
        """

        return await self._client.GetPlayerLoadouts(
            self.platform, self.username, title, **kwargs
        )

    async def loadoutUnlocks(self, title: Title, **kwargs) -> List[LoadoutItem]:
        """
        Get the Call of Duty player's loadout unlocks for the specified title and mode.

        Parameters
        ----------
        title : callofduty.Title
            Call of Duty title to get the player's loadout unlocks from.
        mode: callofduty.Mode, optional
            Call of Duty mode to get the player's loadout unlocks from (default is Multiplayer.)

        Returns
        -------
        list
            Array of loadout item objects.
        """

        return await self._client.GetPlayerLoadoutUnlocks(
            self.platform, self.username, title, **kwargs
        )

    async def authenticityStamp(self, phrase: str, **kwargs):
        """
        Get the Call of Duty player's Authenticity Stamp for the specified phrase.

        Parameters
        ----------
        phrase : str
            Authenticity Stamp code.
        title : callofduty.Title, optional
            Call of Duty title to get the Authenticity Stamp from (default is Black Ops 4.)

        Returns
        -------
        callofduty.AuthenticityStamp
            AuthenticityStamp object representing the requested Authenticity Stamp.
        """

        return await self._client.GetAuthenticityStamp(
            self.platform, self.username, phrase, **kwargs
        )

    async def squad(self):
        """
        Get the Call of Duty player's Squad.

        Returns
        -------
        object
            Squad object for the requested Squad.
        """

        return await self._client.GetPlayerSquad(self.platform, self.username)
